import sys
import os

from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import ThalesRemoteScriptWrapper, PotentiostatMode
from thales_remote.file_interface import ThalesFileInterface

from zahner_analysis.file_import.isc_import import IscImport
from zahner_analysis.file_import.ism_import import IsmImport

import matplotlib.pyplot as plt
import matplotlib as mpl
import numpy as np
from matplotlib.ticker import EngFormatter
import jinja2

if __name__ == "__main__":
    remoteIP = "localhost"

    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm(remoteIP, "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    measDataInterface = ThalesFileInterface(remoteIP)
    measDataInterface.disableSaveReceivedFilesToDisk()
    measDataInterface.enableKeepReceivedFilesInObject()
    measDataInterface.enableAutomaticFileExchange(fileExtensions="*.ism*.isc")

    zahnerZennium.calibrateOffsets()

    zahnerZennium.setCVStartPotential(0)
    zahnerZennium.setCVUpperReversingPotential(1)
    zahnerZennium.setCVLowerReversingPotential(0)
    zahnerZennium.setCVEndPotential(0)

    zahnerZennium.setCVStartHoldTime(2)
    zahnerZennium.setCVEndHoldTime(2)

    zahnerZennium.setCVCycles(1.5)
    zahnerZennium.setCVSamplesPerCycle(400)

    zahnerZennium.setCVMaximumCurrent(1e-3)
    zahnerZennium.setCVMinimumCurrent(-1e-3)

    zahnerZennium.setCVOhmicDrop(0)

    zahnerZennium.disableCVAutoRestartAtCurrentOverflow()
    zahnerZennium.disableCVAutoRestartAtCurrentUnderflow()
    zahnerZennium.disableCVAnalogFunctionGenerator()

    zahnerZennium.setCVNaming("individual")
    zahnerZennium.setCVOutputPath(r"C:\THALES\temp\cv")
    zahnerZennium.setCVOutputFileName("cv_measurement")

    scanRate = 0.25

    zahnerZennium.setCVScanRate(scanRate)

    zahnerZennium.checkCVSetup()
    zahnerZennium.measureCV()

    CVmeasurementData = IscImport(measDataInterface.getLatestReceivedFile().binaryData)
    maximumCurrent = max(CVmeasurementData.getCurrentArray())
    minimumCurrent = min(CVmeasurementData.getCurrentArray())

    capacitance = ((maximumCurrent - minimumCurrent) / 2) / scanRate

    capacitanceFormatter = EngFormatter(places=3, unit="F")
    print(f"{capacitanceFormatter.format_data(capacitance)} Capacitor")

    figCV, (axis) = plt.subplots(1, 1)
    figCV.suptitle(
        f"Cyclic Voltammetry {capacitanceFormatter.format_data(capacitance)} Capacitor"
    )

    axis.plot(
        CVmeasurementData.getVoltageArray(),
        CVmeasurementData.getCurrentArray(),
        color="red",
    )

    axis.grid(which="both")
    axis.xaxis.set_major_formatter(EngFormatter(unit="V"))
    axis.yaxis.set_major_formatter(EngFormatter(unit="A"))
    axis.set_xlabel(r"Voltage")
    axis.set_ylabel(r"Current")

    figCV.set_size_inches(10, 10)
    plt.show()
    figCV.savefig("CV.pdf")

    zahnerZennium.setEISNaming("individual")
    zahnerZennium.setEISOutputPath(r"C:\THALES\temp")
    zahnerZennium.setEISOutputFileName("eis_measurement")

    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
    zahnerZennium.setAmplitude(10e-3)
    zahnerZennium.setPotential(0)
    zahnerZennium.setLowerFrequencyLimit(10)
    zahnerZennium.setStartFrequency(10)
    zahnerZennium.setUpperFrequencyLimit(500000)
    zahnerZennium.setLowerNumberOfPeriods(3)
    zahnerZennium.setLowerStepsPerDecade(5)
    zahnerZennium.setUpperNumberOfPeriods(20)
    zahnerZennium.setUpperStepsPerDecade(5)
    zahnerZennium.setScanDirection("startToMin")
    zahnerZennium.setScanStrategy("single")

    zahnerZennium.enablePotentiostat()
    zahnerZennium.measureEIS()
    zahnerZennium.disablePotentiostat()

    zahnerZennium.setAmplitude(0)

    EISmeasurementData = IsmImport(measDataInterface.getLatestReceivedFile().binaryData)

    zenniumConnection.disconnectFromTerm()
    measDataInterface.close()

    impedanceFrequencies = EISmeasurementData.getFrequencyArray()
    impedanceAbsolute = EISmeasurementData.getImpedanceArray()
    impedancePhase = EISmeasurementData.getPhaseArray()
    figBode, (impedanceAxis) = plt.subplots(1, 1)
    figBode.suptitle(f"EIS {capacitanceFormatter.format_data(capacitance)} Capacitor")

    phaseAxis = impedanceAxis.twinx()

    impedanceAxis.loglog(
        impedanceFrequencies, impedanceAbsolute, marker="o", markersize=3, color="blue"
    )
    impedanceAxis.xaxis.set_major_formatter(EngFormatter(unit="Hz"))
    impedanceAxis.yaxis.set_major_formatter(EngFormatter(unit="$\Omega$"))
    impedanceAxis.set_xlabel(r"f")
    impedanceAxis.set_ylabel(r"|Z|")
    impedanceAxis.yaxis.label.set_color("blue")
    impedanceAxis.grid(which="both")
    impedanceAxis.set_xlim([min(impedanceFrequencies), max(impedanceFrequencies)])

    phaseAxis.semilogx(
        impedanceFrequencies,
        np.abs(impedancePhase * (360 / (2 * np.pi))),
        marker="o",
        markersize=3,
        color="red",
    )
    phaseAxis.yaxis.set_major_formatter(EngFormatter(unit="$°$", sep=""))
    phaseAxis.xaxis.set_major_formatter(EngFormatter(unit="Hz"))
    phaseAxis.set_xlabel(r"f")
    phaseAxis.set_ylabel(r"|Phase|")
    phaseAxis.yaxis.label.set_color("red")
    phaseAxis.set_ylim([0, 90])
    figBode.set_size_inches(10, 4)
    plt.show()
    figBode.savefig("EIS.pdf")

    mpl.rcParams["axes.unicode_minus"] = False
    prefixFormatter = EngFormatter(places=3, sep="")
    defaultMu = prefixFormatter.ENG_PREFIXES[-6]
    prefixFormatter.ENG_PREFIXES[-6] = "\\textmu"  # LaTeX notation for micro

    with open("EIS.csv", "wb") as file:
        file.write(bytearray("Frequency;Impedance;Phase" + os.linesep, "utf-8"))

        for freq, imp, phase in zip(
            impedanceFrequencies, impedanceAbsolute, impedancePhase
        ):
            file.write(
                bytearray(
                    f"{prefixFormatter.format_data(freq)};{prefixFormatter.format_data(imp)};{prefixFormatter.format_data(phase * (360 / (2 * np.pi)))}"
                    + os.linesep,
                    "utf-8",
                )
            )

    objectName = input("Input Test Object Identifier:")

    latex_jinja_env = jinja2.Environment(
        variable_start_string="\PYVAR{",
        variable_end_string="}",
        trim_blocks=True,
        autoescape=False,
        loader=jinja2.FileSystemLoader(os.path.abspath(".")),
    )

    template = latex_jinja_env.get_template(r"report.tex")

    currentFormatter = EngFormatter(unit="A")
    currentFormatter.ENG_PREFIXES[-6] = "\\mu "  # LaTeX notation for micro

    fileString = template.render(
        objectname=objectName,
        capacitance=capacitanceFormatter.format_data(capacitance),
        cv_filename="CV.pdf",
        eis_filename="EIS.pdf",
        eis_csv_filename="EIS.csv",
        cv_maximum_current=currentFormatter.format_data(maximumCurrent),
        cv_minimum_current=currentFormatter.format_data(minimumCurrent),
        cv_scanrate=scanRate,
    )

    fileString = bytearray(fileString, "utf-8")
    f = open("report_filled.tex", "wb")
    f.write(fileString)
    f.close()

    # Only needed for Jupyter, that the kernel does not have to be restarted all the time.
    currentFormatter.ENG_PREFIXES[-6] = defaultMu

    command = f'pdflatex.exe report_filled.tex -jobname="{objectName}"'
    os.system(command)
    os.popen(f"{objectName}.pdf")
    print("finish")



import sys
import time

from delta_remote.connection import DeltaConnection
from delta_remote.script_wrapper import DeltaSCPIWrapper, DeltaSources

from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper
from thales_remote.file_interface import ThalesFileInterface
from zahner_analysis.file_import.ism_import import IsmImport

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import EngFormatter

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()
    zahnerZennium.calibrateOffsets()

    fileInterface = ThalesFileInterface("localhost")
    fileInterface.enableKeepReceivedFilesInObject()
    fileInterface.enableAutomaticFileExchange()

    deltaConnection = DeltaConnection()
    deltaConnection.connect(ip="192.168.2.73", port=8462)

    deltaSM18_220 = DeltaSCPIWrapper(deltaConnection)

    print(f"IDN:\t{deltaSM18_220.IDN()}")

    deltaSM18_220.setProgSourceVoltage(DeltaSources.eth)
    deltaSM18_220.setProgSourceCurrent(DeltaSources.eth)

    zahnerZennium.setEISOutputPath(r"C:\THALES\temp")
    zahnerZennium.setEISNaming("individual")
    zahnerZennium.setEISOutputFileName("spectra_5aac")

    zahnerZennium.selectPotentiostat(1)
    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_GALVANOSTATIC)
    zahnerZennium.setAmplitude(0)
    zahnerZennium.setLowerFrequencyLimit(0.01)
    zahnerZennium.setStartFrequency(1000)
    zahnerZennium.setUpperFrequencyLimit(100000)
    zahnerZennium.setLowerNumberOfPeriods(5)
    zahnerZennium.setLowerStepsPerDecade(5)
    zahnerZennium.setUpperNumberOfPeriods(10)
    zahnerZennium.setUpperStepsPerDecade(10)
    zahnerZennium.setScanDirection("startToMax")
    zahnerZennium.setScanStrategy("single")

    zahnerZennium.setCurrent(0)
    zahnerZennium.enablePotentiostat()

    time.sleep(1.0)

    deltaSM18_220.setTargetVoltage(6)
    deltaSM18_220.setTargetCurrent(20)
    deltaSM18_220.enableOutput()

    time.sleep(1)

    print(f"Measured current\tEL1002:\t\t{zahnerZennium.getCurrent():>10.3f} A")

    print(
        f"Measured current\tDelta SM18-220:\t{deltaSM18_220.getMeasuredCurrent():>10.3f} A"
    )
    print(
        f"Measured voltage\tDelta SM18-220:\t{deltaSM18_220.getMeasuredVoltage():>10.3f} V"
    )
    print(
        f"Measured power\t\tDelta SM18-220:\t{deltaSM18_220.getMeasuredPower():>10.3f} W"
    )

    zahnerZennium.setAmplitude(5)
    zahnerZennium.measureEIS()
    zahnerZennium.setAmplitude(0)

    deltaSM18_220.disableOutput()

    time.sleep(1)

    zahnerZennium.disablePotentiostat()

    ismFile = IsmImport(fileInterface.getLatestReceivedFile().binaryData)

    impedanceFrequencies = ismFile.getFrequencyArray()
    impedanceAbsolute = ismFile.getImpedanceArray()
    impedancePhase = ismFile.getPhaseArray()
    impedanceComplex = ismFile.getComplexImpedanceArray()

    figBode, (impedanceAxis) = plt.subplots(1, 1)

    phaseAxis = impedanceAxis.twinx()

    impedanceAxis.loglog(
        impedanceFrequencies,
        impedanceAbsolute,
        linestyle="dashed",
        linewidth=1,
        marker="o",
        markersize=5,
        fillstyle="none",
        color="blue",
    )
    impedanceAxis.xaxis.set_major_formatter(EngFormatter(unit="Hz"))
    impedanceAxis.yaxis.set_major_formatter(EngFormatter(unit="$\Omega$"))
    impedanceAxis.set_xlabel(r"f")
    impedanceAxis.set_ylabel(r"|Z|")
    impedanceAxis.yaxis.label.set_color("blue")
    impedanceAxis.grid(which="both", linestyle="dashed", linewidth=0.5)
    impedanceAxis.set_xlim(
        [min(impedanceFrequencies) * 0.8, max(impedanceFrequencies) * 1.2]
    )

    phaseAxis.semilogx(
        impedanceFrequencies,
        np.abs(impedancePhase * (360 / (2 * np.pi))),
        linestyle="dashed",
        linewidth=1,
        marker="o",
        markersize=5,
        fillstyle="none",
        color="red",
    )
    phaseAxis.yaxis.set_major_formatter(EngFormatter(unit="$°$", sep=""))
    phaseAxis.xaxis.set_major_formatter(EngFormatter(unit="Hz"))
    phaseAxis.set_xlabel(r"f")
    phaseAxis.set_ylabel(r"|Phase|")
    phaseAxis.yaxis.label.set_color("red")
    phaseAxis.set_ylim([0, 90])
    figBode.set_size_inches(15, 6)
    plt.show()

    zahnerZennium.selectPotentiostat(0)
    zenniumConnection.disconnectFromTerm()
    fileInterface.close()

    deltaSM18_220.setProgSourceVoltage(DeltaSources.front)
    deltaSM18_220.setProgSourceCurrent(DeltaSources.front)
    deltaConnection.disconnect()

    print("finish")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import ThalesRemoteScriptWrapper

from zahner_analysis.file_import.iss_import import IssImport

import matplotlib.pyplot as plt
from matplotlib.ticker import EngFormatter

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.setIEOutputPath(r"C:\THALES\temp\ie")

    zahnerZennium.setIENaming("individual")

    zahnerZennium.calibrateOffsets()

    zahnerZennium.setIEFirstEdgePotential(0)
    zahnerZennium.setIEFirstEdgePotentialRelation("absolute")
    zahnerZennium.setIESecondEdgePotential(0.4)
    zahnerZennium.setIESecondEdgePotentialRelation("absolute")
    zahnerZennium.setIEThirdEdgePotential(-0.4)
    zahnerZennium.setIEThirdEdgePotentialRelation("absolute")
    zahnerZennium.setIEFourthEdgePotential(0)
    zahnerZennium.setIEFourthEdgePotentialRelation("absolute")

    zahnerZennium.setIEPotentialResolution(0.005)
    zahnerZennium.setIEMinimumWaitingTime(0.1)
    zahnerZennium.setIEMaximumWaitingTime(3)
    zahnerZennium.setIERelativeTolerance(0.01)  # 1 %
    zahnerZennium.setIEAbsoluteTolerance(0.001)  # 1 mA
    zahnerZennium.setIEOhmicDrop(0)

    zahnerZennium.setIEScanRate(0.05)
    zahnerZennium.setIEMaximumCurrent(3)
    zahnerZennium.setIEMinimumCurrent(-3)

    zahnerZennium.setIESweepMode("steady state")
    zahnerZennium.setIEOutputFileName("ie_steady")

    zahnerZennium.checkIESetup()
    print(zahnerZennium.readIESetup())

    zahnerZennium.measureIE()

    zahnerZennium.setIESweepMode("dynamic scan")
    zahnerZennium.setIEOutputFileName("ie_dynamic")

    zahnerZennium.checkIESetup()
    print(zahnerZennium.readIESetup())

    zahnerZennium.measureIE()

    zahnerZennium.setIESweepMode("fixed sampling")
    zahnerZennium.setIEOutputFileName("ie_fixed")

    zahnerZennium.checkIESetup()
    print(zahnerZennium.readIESetup())

    zahnerZennium.measureIE()

    zenniumConnection.disconnectFromTerm()

    measurementData = IssImport(r"C:\THALES\temp\ie\ie_steady.iss")

    fig, (axis) = plt.subplots(1, 1)
    axis.semilogy(
        measurementData.getVoltageArray(),
        abs(measurementData.getCurrentArray()),
        color="red",
    )

    axis.grid(which="both")
    axis.xaxis.set_major_formatter(EngFormatter(unit="V"))
    axis.yaxis.set_major_formatter(EngFormatter(unit="A"))
    axis.set_xlabel(r"Voltage")
    axis.set_ylabel(r"Current")
    fig.set_size_inches(10, 10)
    plt.show()

    print("finish")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import (
    PotentiostatMode,
    ThalesRemoteScriptWrapper,
    ScanStrategy,
    ScanDirection,
)

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

    zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test1")
    zahnerZennium.setEISNaming("counter")
    zahnerZennium.setEISCounter(13)
    zahnerZennium.setEISOutputFileName("spectra")

    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
    zahnerZennium.setAmplitude(10e-3)
    zahnerZennium.setPotential(1)
    zahnerZennium.setLowerFrequencyLimit(10)
    zahnerZennium.setStartFrequency(1000)
    zahnerZennium.setUpperFrequencyLimit(10000)
    zahnerZennium.setLowerNumberOfPeriods(5)
    zahnerZennium.setLowerStepsPerDecade(2)
    zahnerZennium.setUpperNumberOfPeriods(20)
    zahnerZennium.setUpperStepsPerDecade(5)
    zahnerZennium.setScanDirection(ScanDirection.START_TO_MAX)
    zahnerZennium.setScanStrategy(ScanStrategy.SINGLE_SINE)

    zahnerZennium.enablePotentiostat()

    zahnerZennium.measureEIS()

    zahnerZennium.disablePotentiostat()

    zahnerZennium.selectPotentiostat(1)

    zahnerZennium.setEISNaming("dateTime")
    zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test2")
    zahnerZennium.setEISOutputFileName("spectra")

    zahnerZennium.setAmplitude(50e-3)
    zahnerZennium.setScanDirection("startToMin")

    zahnerZennium.measureEIS()

    zahnerZennium.setEISNaming("individual")
    zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test3")

    AmplitudesIn_mV_forMeasurement = [5, 10, 20, 50]

    for amplitude in AmplitudesIn_mV_forMeasurement:
        zahnerZennium.setEISOutputFileName("spectraAmplitude{}mV".format(amplitude))
        zahnerZennium.setAmplitude(amplitude / 1000)
        zahnerZennium.measureEIS()

    zahnerZennium.setAmplitude(0)

    zahnerZennium.selectPotentiostat(0)

    zenniumConnection.disconnectFromTerm()
    print("finish")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import ThalesRemoteScriptWrapper

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()
    zahnerZennium.calibrateOffsets()

    zahnerZennium.setCVOutputPath(r"C:\THALES\temp\cv")

    zahnerZennium.setCVOutputFileName("cv_series")
    zahnerZennium.setCVNaming("counter")
    zahnerZennium.setCVCounter(1)

    zahnerZennium.setCVStartPotential(1)
    zahnerZennium.setCVUpperReversingPotential(2)
    zahnerZennium.setCVLowerReversingPotential(0)
    zahnerZennium.setCVEndPotential(1)

    zahnerZennium.setCVStartHoldTime(2)
    zahnerZennium.setCVEndHoldTime(2)

    zahnerZennium.setCVCycles(1.5)
    zahnerZennium.setCVSamplesPerCycle(400)
    zahnerZennium.setCVScanRate(0.5)

    zahnerZennium.setCVMaximumCurrent(0.03)
    zahnerZennium.setCVMinimumCurrent(-0.03)

    zahnerZennium.setCVOhmicDrop(0)

    zahnerZennium.disableCVAutoRestartAtCurrentOverflow()
    zahnerZennium.disableCVAutoRestartAtCurrentUnderflow()
    zahnerZennium.disableCVAnalogFunctionGenerator()

    zahnerZennium.checkCVSetup()
    print(zahnerZennium.readCVSetup())

    zahnerZennium.measureCV()

    zahnerZennium.selectPotentiostat(1)

    zahnerZennium.setCVNaming("individual")
    zahnerZennium.setCVOutputPath(r"C:\THALES\temp\cv")

    ScanRatesForMeasurement = [0.1, 0.2, 0.5, 1.0]

    for scanRate in ScanRatesForMeasurement:
        zahnerZennium.setCVOutputFileName(
            "cv_scanrate_{:d}mVs".format(int(scanRate * 1000))
        )
        zahnerZennium.setCVScanRate(scanRate)

        zahnerZennium.checkCVSetup()
        print(zahnerZennium.readCVSetup())

        zahnerZennium.measureCV()

    zahnerZennium.selectPotentiostat(0)

    zenniumConnection.disconnectFromTerm()
    print("finish")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper

from zahner_analysis.file_import.ism_import import IsmImport
from zahner_analysis.plotting.impedance_plot import nyquistPlotter, bodePlotter

import matplotlib.pyplot as plt
import numpy as np

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

    zahnerZennium.setEISNaming("counter")
    zahnerZennium.setEISCounter(1)
    zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test1")
    zahnerZennium.setEISOutputFileName("spectra")

    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
    zahnerZennium.setAmplitude(10e-3)
    zahnerZennium.setPotential(0)
    zahnerZennium.setLowerFrequencyLimit(0.01)
    zahnerZennium.setStartFrequency(1000)
    zahnerZennium.setUpperFrequencyLimit(200000)
    zahnerZennium.setLowerNumberOfPeriods(3)
    zahnerZennium.setLowerStepsPerDecade(5)
    zahnerZennium.setUpperNumberOfPeriods(20)
    zahnerZennium.setUpperStepsPerDecade(10)
    zahnerZennium.setScanDirection("startToMax")
    zahnerZennium.setScanStrategy("single")

    zahnerZennium.enablePotentiostat()
    zahnerZennium.measureEIS()
    zahnerZennium.disablePotentiostat()

    zahnerZennium.setAmplitude(0)

    zenniumConnection.disconnectFromTerm()

    ismFile = IsmImport(r"C:\THALES\temp\test1\spectra_0001.ism")

    impedanceFrequencies = ismFile.getFrequencyArray()

    impedanceAbsolute = ismFile.getImpedanceArray()
    impedancePhase = ismFile.getPhaseArray()

    impedanceComplex = ismFile.getComplexImpedanceArray()

    print("Measurement end time: " + str(ismFile.getMeasurementEndDateTime()))

    (figNyquist, nyquistAxis) = nyquistPlotter(impedanceObject=ismFile)

    figNyquist.suptitle("Nyquist")
    figNyquist.set_size_inches(18, 18)

    plt.show()

    figNyquist.savefig("nyquist.svg")

    (figBode, (impedanceAxis, phaseAxis)) = bodePlotter(impedanceObject=ismFile)

    figBode.suptitle("Bode")
    figBode.set_size_inches(18, 12)

    plt.show()

    figBode.savefig("bode.svg")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper
import time


if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.disableFraMode()

    zahnerZennium.setFraVoltageMinimum(0)
    zahnerZennium.setFraVoltageMaximum(18)
    zahnerZennium.setFraCurrentMinimum(0)
    zahnerZennium.setFraCurrentMaximum(220)

    zahnerZennium.setFraVoltageInputGain(18.0 / 5.0)
    zahnerZennium.setFraVoltageOutputGain(18.0 / 5.0)
    zahnerZennium.setFraCurrentInputGain(-220.0 / 5.0)
    zahnerZennium.setFraCurrentOutputGain(-220.0 / 5.0)

    zahnerZennium.setFraPotentiostatMode(PotentiostatMode.POTMODE_GALVANOSTATIC)

    zahnerZennium.enableFraMode()
    zahnerZennium.setCurrent(0)

    for i in range(5):
        zahnerZennium.setCurrent(i)
        time.sleep(1)
        print(f"Potential:\t{zahnerZennium.getPotential()}\tV")

    zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test1")
    zahnerZennium.setEISNaming("counter")
    zahnerZennium.setEISCounter(13)
    zahnerZennium.setEISOutputFileName("spectra")

    zahnerZennium.setAmplitude(0.5)
    zahnerZennium.setCurrent(5)
    zahnerZennium.setLowerFrequencyLimit(0.1)
    zahnerZennium.setStartFrequency(10)
    zahnerZennium.setUpperFrequencyLimit(100)
    zahnerZennium.setLowerNumberOfPeriods(3)
    zahnerZennium.setLowerStepsPerDecade(2)
    zahnerZennium.setUpperNumberOfPeriods(20)
    zahnerZennium.setUpperStepsPerDecade(3)
    zahnerZennium.setScanDirection("startToMax")
    zahnerZennium.setScanStrategy("single")

    zahnerZennium.measureEIS()

    zahnerZennium.disableFraMode()

    zenniumConnection.disconnectFromTerm()
    print("finish")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper
import time
import threading

zenniumConnection = None
zahnerZennium = None
zenniumConnectionLiveData = None

keepThreadRunning = True


def watchThreadFunction():
    global keepThreadRunning
    global zenniumConnection
    global zahnerZennium

    print("watch thread started")
    while keepThreadRunning:
        time.sleep(1)
        try:
            beat = zahnerZennium.getWorkstationHeartBeat(2)
        except:
            print("term error watch thread")
            keepThreadRunning = False
        else:
            print("Heartbeat: " + str(beat) + " ms")

    print("watch thread left")
    return


def liveDataThreadFunction():
    global keepThreadRunning
    global zenniumConnectionLiveData

    print("live thread started")
    while keepThreadRunning:
        try:
            data = zenniumConnectionLiveData.waitForBinaryTelegram()
            packetId = data[0]
            data = data[1:]
            """
            Type:
            1 = Init measurement begin
            2 = Measurement end
            4 = Measurement data names
            5 = Measurement data units
            6 = ASCII data
            """
            if packetId in [1, 2, 4, 5, 6]:
                print(data.decode("ASCII"))
        except:
            """
            The connection to the term has an error or the socket has been closed.
            """
            print("term error live thread")
            keepThreadRunning = False

    print("live thread left")
    return


if __name__ == "__main__":
    zenniumConnectionLiveData = ThalesRemoteConnection()
    zenniumConnectionLiveData.connectToTerm("localhost", "Logging")

    liveThread = threading.Thread(target=liveDataThreadFunction)
    liveThread.start()

    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

    watchThread = threading.Thread(target=watchThreadFunction)
    watchThread.start()

    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
    zahnerZennium.setAmplitude(10e-3)
    zahnerZennium.setPotential(0)
    zahnerZennium.setLowerFrequencyLimit(750)
    zahnerZennium.setStartFrequency(1000)
    zahnerZennium.setUpperFrequencyLimit(1500)
    zahnerZennium.setLowerNumberOfPeriods(2)
    zahnerZennium.setLowerStepsPerDecade(2)
    zahnerZennium.setUpperNumberOfPeriods(2)
    zahnerZennium.setUpperStepsPerDecade(20)
    zahnerZennium.setScanDirection("startToMax")
    zahnerZennium.setScanStrategy("single")

    zahnerZennium.enablePotentiostat()

    zahnerZennium.setFrequency(1)
    zahnerZennium.setAmplitude(10e-3)
    zahnerZennium.setNumberOfPeriods(3)

    print("measurement start")

    zahnerZennium.measureEIS()
    for i in range(20):
        zahnerZennium.getPotential()
        zahnerZennium.setPotential(0)

    print("measurement end")

    zahnerZennium.disablePotentiostat()

    print("set thread kill flag")
    keepThreadRunning = False

    print("disconnect connections")
    zenniumConnection.disconnectFromTerm()
    zenniumConnectionLiveData.disconnectFromTerm()

    print("join the threads")
    liveThread.join()
    watchThread.join()

    print("finish")



import sys
import math
import cmath
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper


def printImpedance(impedance):
    print(
        f"Impedance: {abs(impedance):>10.3e} ohm {cmath.phase(impedance)/cmath.pi*180.0:>10.2f} degree"
    )
    return


def spectrum(scriptHandle, lower_frequency, upper_frequency, number_of_points):
    log_lower_frequency = math.log(lower_frequency)
    log_upper_frequency = math.log(upper_frequency)
    log_interval_spacing = (log_upper_frequency - log_lower_frequency) / (
        number_of_points - 1
    )

    for i in range(number_of_points):
        current_frequency = math.exp(log_lower_frequency + log_interval_spacing * i)
        print(f"Frequency: {current_frequency:e} Hz")
        printImpedance(scriptHandle.getImpedance(current_frequency))

    return


TARGET_HOST = "localhost"

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm(TARGET_HOST, "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
    zahnerZennium.setPotential(1.0)
    zahnerZennium.enablePotentiostat()

    for i in range(5):
        print(f"Potential:\t{zahnerZennium.getPotential():>10.6f} V")
        print(f"Current:\t{zahnerZennium.getCurrent():>10.3e} A")

    zahnerZennium.disablePotentiostat()
    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_GALVANOSTATIC)
    zahnerZennium.setCurrent(20e-9)
    zahnerZennium.enablePotentiostat()

    for i in range(5):
        print(f"Potential:\t{zahnerZennium.getPotential():>10.6f} V")
        print(f"Current:\t{zahnerZennium.getCurrent():>10.3e} A")

    zahnerZennium.disablePotentiostat()
    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
    zahnerZennium.setPotential(1.0)
    zahnerZennium.enablePotentiostat()
    zahnerZennium.setFrequency(2000)
    zahnerZennium.setNumberOfPeriods(3)

    zahnerZennium.enablePotentiostat()
    zahnerZennium.getCurrent()

    zahnerZennium.setAmplitude(10e-3)

    printImpedance(zahnerZennium.getImpedance())
    printImpedance(zahnerZennium.getImpedance(2000))
    printImpedance(zahnerZennium.getImpedance(2000, 10e-3, 3))

    spectrum(zahnerZennium, 1000, 2e5, 10)

    zahnerZennium.disablePotentiostat()
    zahnerZennium.setAmplitude(0)

    zenniumConnection.disconnectFromTerm()
    print("finish")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import (
    PotentiostatMode,
    ScanStrategy,
    ScanDirection,
    FileNaming,
    Pad4Mode,
    ThalesRemoteScriptWrapper,
)

from zahner_analysis.file_import.ism_import import IsmImport
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import EngFormatter

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

    zahnerZennium.setEISNaming(FileNaming.COUNTER)
    zahnerZennium.setEISCounter(1)
    zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test1")
    zahnerZennium.setEISOutputFileName("spectra_cells")

    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
    zahnerZennium.setAmplitude(100e-3)
    zahnerZennium.setPotential(0)
    zahnerZennium.setLowerFrequencyLimit(1)
    zahnerZennium.setStartFrequency(1000)
    zahnerZennium.setUpperFrequencyLimit(10000)
    zahnerZennium.setLowerNumberOfPeriods(2)
    zahnerZennium.setLowerStepsPerDecade(5)
    zahnerZennium.setUpperNumberOfPeriods(20)
    zahnerZennium.setUpperStepsPerDecade(10)
    zahnerZennium.setScanDirection(ScanDirection.START_TO_MAX)
    zahnerZennium.setScanStrategy(ScanStrategy.SINGLE_SINE)

    zahnerZennium.setupPad4Channel(1, 1, 1, voltageRange=4.0, shuntResistor=10e-3)
    zahnerZennium.setupPad4Channel(1, 2, 1, voltageRange=4.0, shuntResistor=10e-3)
    zahnerZennium.setupPad4ModeGlobal(Pad4Mode.VOLTAGE)  # or Pad4Mode.CURRENT
    zahnerZennium.enablePad4Global()

    zahnerZennium.enablePotentiostat()
    zahnerZennium.measureEIS()
    zahnerZennium.disablePotentiostat()
    zahnerZennium.setAmplitude(0)
    zenniumConnection.disconnectFromTerm()

    ismFileStack = IsmImport(r"C:\THALES\temp\test1\spectra_cells_0001_ser00.ism")
    impedanceFrequenciesStack = ismFileStack.getFrequencyArray()
    impedanceAbsoluteStack = ismFileStack.getImpedanceArray()
    impedancePhaseStack = ismFileStack.getPhaseArray()
    impedanceComplexStack = ismFileStack.getComplexImpedanceArray()

    ismFileCell1 = IsmImport(r"C:\THALES\temp\test1\spectra_cells_0001_ser01.ism")
    impedanceFrequenciesCell1 = ismFileCell1.getFrequencyArray()
    impedanceAbsoluteCell1 = ismFileCell1.getImpedanceArray()
    impedancePhaseCell1 = ismFileCell1.getPhaseArray()
    impedanceComplexCell1 = ismFileCell1.getComplexImpedanceArray()

    ismFileCell2 = IsmImport(r"C:\THALES\temp\test1\spectra_cells_0001_ser02.ism")
    impedanceFrequenciesCell2 = ismFileCell2.getFrequencyArray()
    impedanceAbsoluteCell2 = ismFileCell2.getImpedanceArray()
    impedancePhaseCell2 = ismFileCell2.getPhaseArray()
    impedanceComplexCell2 = ismFileCell2.getComplexImpedanceArray()

    figNyquist, (nyquistAxis) = plt.subplots(1, 1)
    figNyquist.suptitle("Nyquist")

    nyquistAxis.plot(
        np.real(impedanceComplexStack),
        -np.imag(impedanceComplexStack),
        marker="x",
        markersize=5,
        label="Stack",
    )
    nyquistAxis.plot(
        np.real(impedanceComplexCell1),
        -np.imag(impedanceComplexCell1),
        marker="x",
        markersize=5,
        label="Cell 1",
    )
    nyquistAxis.plot(
        np.real(impedanceComplexCell2),
        -np.imag(impedanceComplexCell2),
        marker="x",
        markersize=5,
        label="Cell 2",
    )

    nyquistAxis.grid(which="both")
    nyquistAxis.set_aspect("equal")
    nyquistAxis.xaxis.set_major_formatter(EngFormatter(unit="$\Omega$"))
    nyquistAxis.yaxis.set_major_formatter(EngFormatter(unit="$\Omega$"))
    nyquistAxis.set_xlabel(r"$Z_{\rm re}$")
    nyquistAxis.set_ylabel(r"$-Z_{\rm im}$")
    nyquistAxis.legend(fontsize="large")
    figNyquist.set_size_inches(20, 8)
    plt.show()
    figNyquist.savefig("nyquist.svg")
